#!/usr/bin/env python
################################################################################
#
# Copyright (c) Members of the EGEE Collaboration. 2004.
# See http://eu-egee.org/partners/ for details on the copyright holders.
# For license conditions see the license file or http://eu-egee.org/license.html
#
################################################################################
# glite-lfc-config v. 1.6.0
#
# Post-installation script for configuring the gLite LHC File Catalog Server
# Alberto Di Meglio < alberto.di.meglio@cern.ch >
#
# Version info: $Id: glite-lfc-config.py,v 1.3 2006/02/11 20:15:26 dimeglio Exp $
#
# Usage: python glite-lfc-config [-c|-v|-h|--help]
#        -c, --checkconf         print configuration
#        -v, --version           print version
#        -h,--help               print usage info
#        --configure             configure the service
#        --ldif      		 update the LCG Providers ldif file and exit
#        --start                 start the service
#        --stop                  stop the service
#        --status                show service status
#
# Return codes: 0 - Ok
#               1 - Configuration failed
#
################################################################################

import os,string,pwd
import sys, posix, getopt,time
import commands

sys.path.append(".")
from gLiteInstallerLib import gLib
from gLiteInstallerLib import ConfigParams 
from  gridSecurity import gridSecurity
from gliteRgmaServicetool import gliteRgmaServicetoolInstance
from gliteRgmaServicetool import gliteRgmaServicetool
import mysql as MySQL

# Set global variables here 
global params                         # all config values from the XML file
global rgmaServicetool

class glite_lfc:

    def __init__(self):
        if params['db.type']=='mysql':	
            self.mysql = MySQL.Mysql()
        self.verbose = 0
        self.version = "1.6.0"
        self.name = "glite-lfc"
        self.friendly_name = "gLite LFC"
        params['module.version'] = self.version
        
    #-------------------------------------------------------------------------------
    # Banner 
    #-------------------------------------------------------------------------------

    def banner(self):

        print "\nxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
        print "Configuring the %s" % self.friendly_name
        print "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n"
    
    #-------------------------------------------------------------------------------
    # Copyright 
    #-------------------------------------------------------------------------------

    def copyright(self):

        print '\nCopyright (c) Members of the EGEE Collaboration. 2004'
        print 'See http://eu-egee.org/partners/ for details on the copyright holders'
        print 'For license conditions see the license file or http://eu-egee.org/license.html'

    #-------------------------------------------------------------------------------
    # Version
    #-------------------------------------------------------------------------------

    def showVersion(self):

        print '\n%s-config  v. %s\n' % (self.name,self.version)
    
    #-------------------------------------------------------------------------------
    # Usage
    #-------------------------------------------------------------------------------

    def usage(self,msg = ""):

        if msg:
            print "\n%s" % (msg)
        
        self.copyright()
        self.showVersion()
    
        print """Usage: \n
Edit the configuration file %s.cfg.xml in
%s/etc.config/templates\n
save it as %s/etc/config/%s.cfg.xml
and run the script as follows\n 
python %s-config [OPTION...]""" % (self.name, os.environ['GLITE_LOCATION'], \
        os.environ['GLITE_LOCATION'], self.name, self.name)

        print '    -c, --checkconf     print the service configuration'
        print '    -v, --version       print the version of the configuration script'
        print '    -h, --help          print this usage information'
        print '    --configure         configure the service'
        print '    --ldif              update the LCG Providers ldif file and exit'
        print '    --start             start the service'
        print '    --stop              stop the service'
        print '    --status            check service status'
        print '\n'

    #-------------------------------------------------------------------------------
    # All the configuration code goes here
    #-------------------------------------------------------------------------------
    def backup(self,path):
		pathBak = "%s.1" %path
		if os.path.exists(pathBak):
			os.remove(pathBak)
		if os.path.exists(path):
			os.rename(path,pathBak)
			
    def createTnsNames(self):
		path = "%s/etc/tnsnames.ora" %os.environ['GLITE_LOCATION']
		params['db.tnsadmin']= "%s/etc" %os.environ['GLITE_LOCATION']
		
		print "Creating tnsnames file under %s" %path	
		self.backup(path)
		file = open(path, 'w')
		file.write("%s=(DESCRIPTION=(ADDRESS=(PROTOCOL=%s)(HOST=%s)(PORT=%s))(CONNECT_DATA=(SID=%s)))\n" \
		%(params['db.name'],params['db.protocol'],params['db.host'],params['db.port'],params['db.sid']))
		file.close()
		
    def existsOracleSchema(self):
		print "Checking Oracle schema"
		path = "check_schema.sql"
		self.backup(path)

		file = open(path,"w")
		file.write("WHENEVER SQLERROR exit failure;\n")
		file.write("select * from schema_version;\n")
		file.write("exit success;")
		file.close()
		
		command = "sqlplus -S %s/%s @check_schema.sql" %(params['db.lfc.user'],params['db.lfc.password'])

		file = os.popen(command, 'r')
		version = file.read()
		if file.close() is None:
			print "There is already a schema loaded into the database",
			glib.printOkMessage()
			return "true"
		else:
			return "false"
			
    def checkOracle(self):
	
		#############################################
		#Checking Oracle requirements
		#############################################
		if params.has_key('oracle-instantclient.location'):
			if params['oracle-instantclient.location'] != "":
				if not os.path.exists(params['oracle-instantclient.location']):
					glib.printErrorMessage("The Oracle InstantClient cannot be found at the specified location")
					glib.printErrorMessage(params['oracle-instantclient.location'])
					glib.printErrorMessage("Configuration cannot continue")
					return 1
			else:
				glib.printErrorMessage("The Oracle InstantClient location must be defined in oracle-instantclient.location")
				return 1
		else:
			glib.printErrorMessage("The Oracle InstantClient configuration variable missing: oracle-instantclient.location")
			return 1
			
		if params['db.tnsadmin'] == "":
			self.createTnsNames()			
			if not os.path.exists("%s/tnsnames.ora" %params['db.tnsadmin']):
				glib.printErrorMessage("The tnsnames.ora file under %s can not be found" %params['db.tnsadmin'])
				return 1
			else:
				print "File tnsnames.ora correctly created under %s" %params['db.tnsadmin'],
				glib.printOkMessage()
		return 0
			
    def start(self,service='all'):

        error_level = 0
        
        #-------------------------------------------------------------------
        # Start mysql
        #-------------------------------------------------------------------
        if params['db.type']=='mysql':
            if service == 'all' or service == 'mysql':
                self.mysql.start()

        #-------------------------------------------------------------------
        # Start lfc
        #-------------------------------------------------------------------
        if service == 'all' or service == 'lfc':
            status_output = commands.getstatusoutput("service lfcdaemon start")
            status = status_output[0]/256
            output = status_output[1]
            print output
            if status != 0:
                glib.printErrorMessage("Failed to start the LFC Server daemon")
                error_level = 1

        #-------------------------------------------------------------------
        # Start dli
        #-------------------------------------------------------------------
        if service == 'all' or service == 'dli':
            status_output = commands.getstatusoutput("service lfc-dli start")
            status = status_output[0]/256
            output = status_output[1]
            print output
            if status != 0:
                glib.printErrorMessage("Failed to start the LFC DLI service")
                error_level = 1

        #-------------------------------------------------------------------
        # Start Servicetool
        #-------------------------------------------------------------------
        if service == 'all' or service == 'st':
            if params['rgma.servicetool.activate'] == "true":
                if rgmaServicetool.start():
                    error_level = 1
					
        #-------------------------------------------------------------------
        # Start R-GMA gin
        #-------------------------------------------------------------------
        if service == 'all':
            print "Starting the gLite R-GMA GIN service ... "
                       
            errorcode = os.system("%s/etc/config/scripts/glite-rgma-gin-config.py --start --subservice" % os.environ['GLITE_LOCATION'])
            if errorcode:
                print "Starting the gLite R-GMA GIN service                         ",                                                
                glib.printFailedMessage()
                error_level = 1

            print "Starting the gLite R-GMA GIN service                         ",                                                
            glib.printOkMessage()			
        
        return error_level
        
    def stop(self,service='all'):

        error_level = 0
        
        #-------------------------------------------------------------------
        # Stop dli
        #-------------------------------------------------------------------
        if service == 'all' or service == 'dli':
            if os.system("service lfc-dli status") == 0:
                status_output = commands.getstatusoutput("service lfc-dli stop")
                status = status_output[0]/256
                output = status_output[1]
                if status != 0:
                    glib.printErrorMessage("Failed to stop the LFC DLI service")
                    error_level = 1

        #-------------------------------------------------------------------
        # Stop lfc
        #-------------------------------------------------------------------
        if service == 'all' or service == 'lfc':
            if os.system("service lfcdaemon status") == 0:
                status_output = commands.getstatusoutput("service lfcdaemon stop")
                status = status_output[0]/256
                output = status_output[1]
                if status != 0:
                    glib.printErrorMessage("Failed to stop the LFC Server daemon")
                    error_level = 1

        #-------------------------------------------------------------------
        # Stop mysql
        #-------------------------------------------------------------------
        if params['db.type']=='mysql':
            if service == 'all' or service == 'mysql':
                self.mysql.stop()

        #--------------------------------------------------------        
        # Stop the servicetool
        #--------------------------------------------------------                
        if service == 'all' or service == 'st':
            if params['rgma.servicetool.activate'] == "true":
                if rgmaServicetool.stop():
                    error_level = 1
					
        #--------------------------------------------------------        
        # Stop R-GMA gin
        #--------------------------------------------------------        
        print "Stopping the gLite R-GMA GIN service ... "

        errorcode = os.system("%s/etc/config/scripts/glite-rgma-gin-config.py --stop --subservice" % os.environ['GLITE_LOCATION'])
        if errorcode:
            print "Stopping the gLite R-GMA GIN service                         ",
            glib.printFailedMessage()
            return 1

        print "Stopping the gLite R-GMA GIN service                         ",
        glib.printOkMessage()			
        
        return error_level
        
    def status(self):

        error_level = 0

        #-------------------------------------------------------------------
        # lfc status
        #-------------------------------------------------------------------
        status_output = commands.getstatusoutput("service lfcdaemon status")
        status = status_output[0]/256
        output = status_output[1]
        if status != 0:
            glib.printErrorMessage("The LFC Server daemon is not running")
            error_level = 1
        else:
            glib.printOkMessage(),
            print("The LFC Server daemon is running")

        #-------------------------------------------------------------------
        # dli status
        #-------------------------------------------------------------------
        status_output = commands.getstatusoutput("service lfc-dli status")
        status = status_output[0]/256
        output = status_output[1]
        if status != 0:
            glib.printErrorMessage("The LFC DLI service is not running")
            error_level = 1
        else:
            glib.printOkMessage(),
            print("The LFC DLI service is running")

        return error_level
        
    def configure(self):
        
        error_level = 0
        if params['db.type'] == 'oracle':
            if self.checkOracle() != 0:
                glib.printErrorMessage("Problems to check the Oracle installation")
                error_level = 1			
				
        #--------------------------------------------------------
        # LFC Services user and group
        #--------------------------------------------------------        

        lfcuid,lfcgid = glib.add_user(params['lfc.user'],params['lfc.group'])
        glib.setUserEnv(params['lfc.user'])
        print "\nCreating/verifying the LFC Manager user name and group             ",
        if lfcuid != 0:
            glib.printOkMessage()
        else:
            glib.printFailedMessage()
            return 1
            
        #--------------------------------------------------------
        # Security settings
        #--------------------------------------------------------        

        lfckeypath = "/etc/grid-security/lfcmgr"
        if not os.path.exists(lfckeypath):
            os.makedirs(lfckeypath)
        glib.check_dir(lfckeypath,0,lfcuid,lfcgid)
        
        grimapdir = params['host.gridmap.dir']
        if not os.path.exists(grimapdir):
            os.makedirs(grimapdir)
        glib.check_dir(grimapdir,1774,0,lfcgid)

        # Copy host certificates to the right place
        os.system("cp %s %s/lfccert.pem" % (params['host.certificate.file'], lfckeypath))
        os.system("cp %s %s/lfckey.pem" % (params['host.key.file'], lfckeypath))

        glib.check_owner("%s/lfccert.pem" % lfckeypath, params['lfc.user'],params['lfc.group'])
        glib.check_perms("%s/lfccert.pem" % lfckeypath, 0644)
        glib.check_owner("%s/lfckey.pem" % lfckeypath, params['lfc.user'],params['lfc.group'])
        glib.check_perms("%s/lfckey.pem" % lfckeypath, 0400)

        print "\nCreating/verifying the LFC Services certificate and key        ",
        glib.printOkMessage()
        
        #-------------------------------------------------------------------------------
        # Update the $GLITE_LOCATION/etc/glite-mkgridmap.conf file if required
        #-------------------------------------------------------------------------------
        
        ret_val = gsec.configure_mkgridmap()
        if ret_val == 1:
            return 1
        elif ret_val == 2:
            error_level = 2

        #-------------------------------------------------------------------------------
        # Configure pool accounts
        #-------------------------------------------------------------------------------
        
        ret_val = gsec.create_pool_accounts()
        if ret_val == 1:
            return 1

        #--------------------------------------------------------
        # Installs the Security Utilities
        #--------------------------------------------------------
        
        if os.system("python %s/glite-security-utils-config.py --subservice" % glib.getScriptPath()):
            print "\nInstalling gLite Security Utilities                   ",
            glib.printFailedMessage()
            return 1

        print "\nInstalling gLite Security Utilities                   ",
        glib.printOkMessage()
        
        #--------------------------------------------------------
        # MySQL Configuration
        #--------------------------------------------------------        

        # start MySQL
        if params['db.type']=='mysql':
            self.mysql.start()                

            # Set root password
            mysql_root_password = params['mysql.root.password']
            if params['set.mysql.root.password'] == "true" and mysql_root_password != "":
                # Check if pwd is empty
                if self.mysql.checkPasswordSilently("") == 0:
                    # Set pwd
                    self.mysql.setpassword(mysql_root_password)

        #--------------------------------------------------------
        # LFC configuration
        #--------------------------------------------------------        

        # Trusted hosts
        if params.has_key('trusted.hosts'):
            
            if not os.path.exists("/etc/shift.conf"):
                os.system("/bin/touch /etc/shift.conf")
                
            infile = open('/etc/shift.conf','r')
            for line in infile:
                if line.find("LFC TRUST") != -1:
                    break
            else:
                infile.close()
                infile = open('/etc/shift.conf','a')
                infile.write("LFC TRUST\n")
                
            infile.close()

            infile = open('/etc/shift.conf','r')
            outfile = open('/etc/shift.conf.1','w')
            for line in infile:
                if line.find("LFC TRUST") == -1:
                    outfile.write(line)
                else:
                    line = "LFC TRUST"
                    for host in params['trusted.hosts']:
                        line = line.strip() + " " + host.strip()
                    line = line + "\n"
                    outfile.write(line)
                 
            infile.close()
            outfile.close()
            
            os.rename('/etc/shift.conf.1','/etc/shift.conf')
                
            print "\nConfiguring trusted hosts                                  ",
            glib.printOkMessage()
    
        # MySQL database setup
        lfcuser = params['db.lfc.user']
        lfcpwd = params['db.lfc.password']
		
        if params['db.type']=='mysql':
            # Check if database exists
            if self.mysql.existsDB("cns_db",mysql_root_password) != 0:
            
                # Create database
                print ('\n==> Creating MySQL cns_db database\n')
    
                if os.path.exists('/bin/rm /tmp/mysql_ct'):
                    os.remove('/tmp/mysql_ct')
            
                file = open('/tmp/mysql_ct', 'w')

                text = ['\. %s/share/LFC/create_lfc_tables_mysql.sql\n' % os.environ['LCG_LOCATION']]
    
                file.writelines(text)
                file.close()
                os.system('/usr/bin/mysql -p%s < /tmp/mysql_ct' % mysql_root_password)
                os.system('/bin/rm /tmp/mysql_ct')

                self.mysql.add_user("cns_db",lfcuser,lfcpwd,mysql_root_password)
        else:
            if params['db.type'] == 'oracle':
                if self.existsOracleSchema() == "false":
                    command = "sqlplus -S %s/%s < %s/share/LFC/db-deployment/create_lfc_tables_oracle.sql" %(params['db.lfc.user'],params['db.lfc.password'],os.environ['LCG_LOCATION'])
                    print "Executing command %s " %command	
                    os.system(command)
            
        # Configuration file
        if not os.path.exists("%s/etc/NSCONFIG" % os.environ['LCG_LOCATION']):
            os.system("/bin/touch %s/etc/NSCONFIG" % os.environ['LCG_LOCATION'])
            
        glib.check_file("%s/etc/NSCONFIG" % os.environ['LCG_LOCATION'],0660,lfcuid,lfcgid)
        file = open("%s/etc/NSCONFIG" % os.environ['LCG_LOCATION'],'w')
        if params['db.type']=='oracle':
            file.write("%s/%s@%s\n" %(params['db.lfc.user'],params['db.lfc.password'],params['db.name']))
            file.write("export ORACLE_HOME=\"%s\"\n" % params['oracle-instantclient.location'])
            file.write("export TNS_ADMIN=\"%s\"\n" % params['db.tnsadmin'])
            file.write("export TWO_TASK=\"%s\"\n" % params['db.name'])
        else:
            file.write("%s/%s@%s\n" % (lfcuser,lfcpwd,glib.fq_hostname))
        file.close()
        
        self.backup("/etc/sysconfig/lfcdaemon")
        file = open("/etc/sysconfig/lfcdaemon","w")
        file.write("RUN_LFCDAEMON=\"yes\"\n")
        if params['db.type']=='oracle':
            file.write("export ORACLE_HOME=\"%s\"\n" % params['oracle-instantclient.location'])
            file.write("export TNS_ADMIN=\"%s\"\n" % params['db.tnsadmin'])
            file.write("export TWO_TASK=\"%s\"\n" % params['db.name'])
        file.close()	
		
        print "\nCreate/Verify the LFC replica database                         ",
        glib.printOkMessage()
		
        # Create initial file system
        self.start('lfc')
        status_output = commands.getstatusoutput("%s/bin/lfc-ls /" % os.environ['LCG_LOCATION'])
        status = status_output[0]/256
        output = status_output[1]
        if output.find('grid') == -1:
            print "Creating 'grid' directory                                    ",
            if not os.system("%s/bin/lfc-mkdir /grid" % os.environ['LCG_LOCATION'])/256:
                glib.printOkMessage()
            else:
                glib.printFailedMessage()
            
        # Create one directory per VO
        volist = glib.getVoList()
        for voItem in volist:
            voparams = glib.getVoParams(voItem)
            vo = voparams['voms.voname']
            if vo.strip() != "":
                status_output = commands.getstatusoutput("%s/bin/lfc-ls /grid" % os.environ['LCG_LOCATION'])
                status = status_output[0]/256
                output = status_output[1]
                if output.find('%s' % vo) == -1:
                    print "Creating %s directory                                    " % vo,
                    ret_val = os.system("%s/bin/lfc-mkdir /grid/%s" % (os.environ['LCG_LOCATION'],vo))/256
                    if not ret_val:
                        os.system("%s/bin/lfc-chown root:%s /grid/%s" % (os.environ['LCG_LOCATION'],vo,vo))
                        os.system("%s/bin/lfc-chmod 775 /grid/%s" % (os.environ['LCG_LOCATION'],vo))
                        os.system("%s/bin/lfc-setacl -m d:u::7,d:g::7,d:o:5 /grid/%s" % (os.environ['LCG_LOCATION'],vo))
                        glib.printOkMessage()
                    else:
                        glib.printFailedMessage()

        #--------------------------------------------------------
        # DLI configuration
        #--------------------------------------------------------        

        if not os.path.exists("/etc/sysconfig/lfc-dli"):
            os.system("cp -p /etc/sysconfig/lfc-dli.templ /etc/sysconfig/lfc-dli")
            
        infile = open("/etc/sysconfig/lfc-dli",'r')
        outfile = open("/etc/sysconfig/lfc-dli.1",'w')
        for line in infile:
            if line.find('export LFC_HOST') == -1:
                outfile.write(line)
            else:
                outfile.write("export LFC_HOST=%s\n" % glib.fq_hostname)
        infile.close()
        outfile.close()						
                
        os.rename("/etc/sysconfig/lfc-dli.1","/etc/sysconfig/lfc-dli")

        #--------------------------------------------------------
        # RGMA servicetool: configure servicetool
        #--------------------------------------------------------  
        if self.configureServiceTool() == 1:
            return 1
		
        # Generate the LFC LCG Info Provider configuration file
        self.generateLFCInfoProvidersFile()		
		
        #--------------------------------------------------------
        # R-GMA gin: configure R-GMA gadget-in tool
        #--------------------------------------------------------
        if self.verbose:
            print "Configuring R-GMA GIn ..."
        
        errorcode = os.system("%s/etc/config/scripts/glite-rgma-gin-config.py --subservice --configure" %os.environ['GLITE_LOCATION'])
        if errorcode:
            print "Configuring R-GMA GadgetIn                                   ",                                                
            glib.printFailedMessage()
            error_level = 1
        else:
            print "Configuring R-GMA GadgetIn                                   ",
            glib.printOkMessage()

        return error_level
		
    def configureServiceTool(self):
		serviceTypes = ["lcg-file-catalog","data-location-interface"]
		serviceScripts = ["/etc/init.d/lfcdaemon status","/etc/init.d/lfc-dli status"]
		
		if params['rgma.servicetool.activate'] == "true":
			for idx in xrange(len(serviceTypes)):
				serviceId = "%s_%s" % (glib.fq_hostname,serviceTypes[idx])
				servicetoolInstance = gliteRgmaServicetoolInstance(glib, serviceId)
				glib.printInfoMessage("Configuring servicetool instance %s" %servicetoolInstance)
                                                                                                                                                               
				# set params
				servicetoolInstance.setServiceName(serviceId)
				servicetoolInstance.setServiceType(serviceTypes[idx])
				servicetoolInstance.setServiceVersion(self.version)
				servicetoolInstance.setStatusScript(serviceScripts[idx])
				servicetoolInstance.setEndpoint('not available')

				# add instance to the gLite configuration
				if servicetoolInstance.add():
					return 1
			
			if rgmaServicetool.configure(glib):
				# error in configuring servicetool
				return 1
		return 0	
		
    def generateLFCInfoProvidersFile(self):
		print "Generating LFC info provider file"
		
		lfcparamsfile = "%s/etc/lcg-info-generic-lfc.conf" % os.environ['GLITE_LOCATION']
		self.backup(lfcparamsfile)
		f = open(lfcparamsfile, 'w')
		f.write("ldif_file=%s/etc/out-lfc.ldif\n" % os.environ['GLITE_LOCATION'])
		f.write("generic_script=%s/libexec/lcg-info-generic\n" % params['lcg.providers.location'])
		f.write("wrapper_script=%s/libexec/lcg-info-wrapper\n" %params['lcg.providers.location'])
		
		f.write("temp_path=%s\n" %os.environ['GLITE_LOCATION_TMP'])
		f.write("template=%s/etc/GlueService.template\n" % params['lcg.providers.location'])

		f.write("# Common for all\n")
		f.write("GlueInformationServiceURL: undefined\n")
		f.write("\n")
		
		f.write("#LFC\n")
		f.write("dn: GlueServiceUniqueID=http://%s:8085/,mds-vo-name=local,o=grid\n" %os.environ["HOSTNAME"]) 
		f.write("GlueServiceName: %s-lfc-dli\n" %params['rgma.servicetool.siteId'])
		f.write("GlueServiceType: data-location-interface\n")
		f.write("GlueServiceVersion: 1\n")
		f.write("GlueServiceEndpoint: http://%s:8085/\n" %os.environ["HOSTNAME"])
		f.write("GlueServiceURI: http://%s:8085/\n" %os.environ["HOSTNAME"])
		f.write("GlueServiceWSDL: not available\n")
		f.write("GlueServiceSemantics: not available\n")
		f.write("GlueServiceAccessPointURL: http://%s:8085/\n" %os.environ["HOSTNAME"]) 
		f.write("GlueServiceStatus: running\n")
		f.write("GlueForeignKey: GlueSiteUniqueID=%s\n" %params['rgma.servicetool.siteId'])
		
		volist = glib.getVoList()
		for voItem in volist:
			voparams = glib.getVoParams(voItem)
			vo = voparams['voms.voname']
			f.write("GlueServiceOwner: %s\n" %vo)
			f.write("GlueServiceAccessControlRule: %s\n" %vo)

		f.write("\n")		
		f.write("dn: GlueServiceUniqueID=%s,mds-vo-name=local,o=grid\n" %os.environ["HOSTNAME"])
		f.write("GlueServiceName: %s-lfc\n" %params['rgma.servicetool.siteId'])
		f.write("GlueServiceType: lcg-file-catalog\n")
		f.write("GlueServiceVersion: 1\n")
		f.write("GlueServiceEndpoint: %s\n" %os.environ["HOSTNAME"])
		f.write("GlueServiceURI: %s\n" %os.environ["HOSTNAME"])
		f.write("GlueServiceWSDL: not available\n")
                f.write("GlueServiceSemantics: not available\n")
		f.write("GlueServiceAccessPointURL: %s\n" %os.environ["HOSTNAME"])
		f.write("GlueServiceStatus: running\n")
		f.write("GlueForeignKey: GlueSiteUniqueID=%s\n" %params['rgma.servicetool.siteId'])
		
		for voItem in volist:
			voparams = glib.getVoParams(voItem)
			vo = voparams['voms.voname']
			f.write("GlueServiceOwner: %s\n" %vo)
			f.write("GlueServiceAccessControlRule: %s\n" %vo)

		f.close()	
		# Load configuration in LCG Info Providers
		os.system("%s/sbin/lcg-info-generic-config %s/etc/lcg-info-generic-lfc.conf" \
			% (params['lcg.providers.location'],os.environ['GLITE_LOCATION']))

		f = open("%s/libexec/lcg-info-wrapper" %params['lcg.providers.location'], 'w')
		f.write("#!/bin/bash\n")
		f.write("%s/libexec/lcg-info-generic %s" %(params['lcg.providers.location'],lfcparamsfile))
		f.close()

		os.chmod("%s/libexec/lcg-info-wrapper" %params['lcg.providers.location'], 0755)
		os.chmod("%s/etc/out-lfc.ldif" % os.environ['GLITE_LOCATION'], 0644)
		
		print "Configuring LFC information provider                          ",
		glib.printOkMessage()
		return 0
        
#-------------------------------------------------------------------------------
# Set all environment variables
#-------------------------------------------------------------------------------

def set_env():

    # gLite
    glib.export('GLITE_LOCATION');
    glib.export('GLITE_LOCATION_VAR');
    if not os.path.exists(os.environ['GLITE_LOCATION_VAR']):
        os.mkdir(os.environ['GLITE_LOCATION_VAR'],0755)
    glib.export('GLITE_LOCATION_LOG');
    if not os.path.exists(os.environ['GLITE_LOCATION_LOG']):
        os.mkdir(os.environ['GLITE_LOCATION_LOG'],0755)
    glib.export('GLITE_LOCATION_TMP');
    if not os.path.exists(os.environ['GLITE_LOCATION_TMP']):
        os.mkdir(os.environ['GLITE_LOCATION_TMP'],0755)

    glib.export('GLOBUS_LOCATION');
    glib.export('LCG_LOCATION');
    glib.export('LFC_HOST',glib.fq_hostname)

    # bin and lib paths
    glib.addEnvPath("PATH","$GLOBUS_LOCATION/bin")
    glib.addEnvPath("PATH","$LCG_LOCATION/bin")
    glib.addEnvPath("PATH","$GLITE_LOCATION/bin")

    glib.addEnvPath("LD_LIBRARY_PATH","$GLOBUS_LOCATION/lib")
    glib.addEnvPath("LD_LIBRARY_PATH","$LCG_LOCATION/lib")
    glib.addEnvPath("LD_LIBRARY_PATH","$GLITE_LOCATION/lib")
    glib.addEnvPath("LD_LIBRARY_PATH","/usr/local/lib")

    glib.exportToLdSoConf("%s/lib" % os.environ['GLOBUS_LOCATION'])
    glib.exportToLdSoConf("/usr/local/lib")
	
    if params['db.type'] == 'oracle':
        glib.addEnvPath("LD_LIBRARY_PATH","%s/lib" %params['oracle-instantclient.location'])
        glib.export("TWO_TASK","%s" %params['db.name'])
        glib.export("ORACLE_HOME",params['oracle-instantclient.location'])
        if params['db.tnsadmin']=="":
            glib.export("TNS_ADMIN","%s/etc" %os.environ['GLITE_LOCATION'])
        else:
            glib.export("TNS_ADMIN","%s" %params['db.tnsadmin'])

    # Set environment
    glib.setUserEnv()
    
#-------------------------------------------------------------------------------
#  Main program begins here 
#-------------------------------------------------------------------------------

if __name__ == '__main__':

    # The script must be run as root
    if not os.geteuid()==0:
        print '"\nThis script must be run as root\n'
        sys.exit(1)
        
    # Get an instance of the library class
    glib = gLib()
    
    # Load parameters
    params = ConfigParams()
    loadDefaults(params)
    try:
        opts, args = glib.getopt(sys.argv[1:], '', ['siteconfig='])
        for o, a in opts:
            if o == "--siteconfig":
                params['site.config.url'] = a
                break
    except getopt.GetoptError:
        pass
    if glib.loadConfiguration(["%s/../glite-lfc.cfg.xml" % glib.getScriptPath(),"%s/../glite-rgma-servicetool.cfg.xml" % glib.getScriptPath()], params):
        print "An error occurred while configuring the service"
        sys.exit(1)
    
    verbose = 0
    if params.has_key('glite.installer.verbose'):
        if params['glite.installer.verbose'] == "true":
            verbose = 1
    glib.verbose = verbose
    
    # Set up the environment
    set_env()
          
    #Instantiate grdSecurity classes
    gsec = gridSecurity(glib)
    gsec.setVerbosity(verbose)
    gsec.params = params

    # Instantiate the service classes
    service = glite_lfc()
    service.verbose = verbose
    # Instantiate the rgma servicetool class
    rgmaServicetool = gliteRgmaServicetool()
    rgmaServicetool.verbose = verbose	
   
    # Command line opts if any
    try:
        opts, args = glib.getopt(sys.argv[1:], 'chv', ['checkconf', 'help', 'version','configure','ldif','stop','start','status','siteconfig='])
    except getopt.GetoptError:
        service.usage(msg = "Unknown options(s)")
        sys.exit(1)

    if len(opts) == 0:
        service.usage()
        sys.exit(0)
    
    # Check cli options
    for o, a in opts:
        if o in ("-h", "--help"):
            service.usage()
            sys.exit(0)

        if o in ("-v", "--version"):
            service.showVersion()
            sys.exit(0)

        if o in ("-c", "--checkconf"):
            service.copyright()
            service.showVersion()
            glib.print_params(params)
            sys.exit(0)

        if o == "--configure":

            # Check certificates
            if params.has_key('glite.installer.checkcerts'):
                if params['glite.installer.checkcerts'] == "true":
                    if glib.check_certs(params) != 0:
                        print "An error occurred while configuring the %s service" \
                            % service.friendly_name
                        sys.exit(1)
            
            # Print configuration parameters
            if verbose:
                glib.print_params(params)
        
            service.copyright()
            service.showVersion()
            service.banner()
                
            # Stop all services
            glib.printInfoMessage("\n\nStopping all running LFC services...")
            service.stop()
            
            # Configure the service
            return_result = service.configure()
            
            if return_result == 0:

                # Stop all services
                glib.printInfoMessage("\n\nStopping all running LFC services...")
                service.stop()
                
                print "\n\nThe %s configuration was successfully completed\n" % service.friendly_name
                print "You can now start the service using the --start option of this script\n\n"
                glib.registerService()

                sys.exit(0)

            elif return_result == 2:

                # Stop all services
                glib.printInfoMessage("\n\nStopping all running LFC services...")
                service.stop()
                
                print "\n\nThe %s configuration was completed,\n" % service.friendly_name
                print "but warnings were issued. Please revise them and re-run the script\n"
                print "or configure LFC manually\n"

                sys.exit(2)

            else:
                print "\n\nAn unrecoverable error occurred while configuring the %s" \
                    % service.friendly_name

                sys.exit(1)
				
        if o == "--ldif":
            service.copyright()
            service.showVersion()
            sys.exit(service.generateLFCInfoProvidersFile())
            
        if o in ("start", "--start"):
            # Start the service
            if service.start() == 0:
                print "\n\nThe %s was successfully started           " % service.friendly_name,
                glib.printOkMessage()
                sys.exit(0)
            else:
                print "\n\nAn error occurred while starting the %s            " % service.friendly_name,
                glib.printFailedMessage()
                sys.exit(1)
            
        if o in ("stop", "--stop"): 
            # Stop the service
            if service.stop() == 0:
                print "\n\nThe %s was successfully stopped           " % service.friendly_name,
                glib.printOkMessage()
                sys.exit(0)
            else:
                print "\n\nAn unrecoverable error occurred while stopping the %s " % service.friendly_name,
                glib.printFailedMessage()
                sys.exit(1)
        
        if o == "--status":
            sys.exit(service.status())
